// SPDX-License-Identifier: Apache-2.0
// Copyright 2025 Letter Combinations Solution
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include "LetterCombinations.h"
#include <iostream>

// Main function for standalone execution
int main() {
    LetterCombinations solution;
    solution.RunDemo();
    
    // Interactive mode
    std::cout << "\n=== Interactive Mode ===" << std::endl;
    std::cout << "Enter digits (2-9) or 'quit' to exit: ";
    
    std::string input;
    while (std::cin >> input && input != "quit") {
        auto combinations = solution.GetCombinations(input);
        solution.PrintCombinations(combinations);
        std::cout << "Total combinations: " << combinations.size() << std::endl;
        std::cout << "\nEnter digits (2-9) or 'quit' to exit: ";
    }
    
    return 0;
}